@@ -1,6 +1,6 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2010 Emmanuel Pacaud
+ * Copyright © 2009-2019 Emmanuel Pacaud
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,24 +28,27 @@
 #endif
 
 #include <arvbuffer.h>
-#include <arvgvsp.h>
+#include <arvgvspprivate.h>
 
 G_BEGIN_DECLS
 
 struct _ArvBufferPrivate {
 	size_t size;
 	gboolean is_preallocated;
-	void *data;
+	unsigned char *data;
 
 	void *user_data;
 	GDestroyNotify user_data_destroy_func;
 
 	ArvBufferStatus status;
 
-	ArvGvspPayloadType gvsp_payload_type;
+	ArvBufferPayloadType payload_type;
+
+	guint32 chunk_endianness;
 
 	guint32 frame_id;
 	guint64 timestamp_ns;
+	guint64 system_timestamp_ns;
 
 	guint32 x_offset;
 	guint32 y_offset;
