@@ -1,6 +1,6 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2010 Emmanuel Pacaud
+ * Copyright © 2009-2019 Emmanuel Pacaud
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 
 #include <arvgcport.h>
 #include <arvgcregisterdescriptionnode.h>
+#include <arvgcfeaturenodeprivate.h>
 #include <arvdevice.h>
 #include <arvgvdevice.h>
 #include <arvchunkparserprivate.h>
@@ -35,12 +36,12 @@
 #include <arvgc.h>
 #include <memory.h>
 
-static GObjectClass *parent_class = NULL;
-
 struct _ArvGcPortPrivate {
 	ArvGcPropertyNode *chunk_id;
 };
 
+G_DEFINE_TYPE_WITH_CODE (ArvGcPort, arv_gc_port, ARV_TYPE_GC_FEATURE_NODE, G_ADD_PRIVATE(ArvGcPort))
+
 /* ArvDomNode implementation */
 
 static const char *
@@ -62,11 +63,11 @@ _post_new_child (ArvDomNode *self, ArvDomNode *child)
 				node->priv->chunk_id = property_node;
 				break;
 			default:
-				ARV_DOM_NODE_CLASS (parent_class)->post_new_child (self, child);
+				ARV_DOM_NODE_CLASS (arv_gc_port_parent_class)->post_new_child (self, child);
 				break;
 		}
 	} else
-		ARV_DOM_NODE_CLASS (parent_class)->post_new_child (self, child);
+		ARV_DOM_NODE_CLASS (arv_gc_port_parent_class)->post_new_child (self, child);
 }
 
 static void
@@ -108,13 +109,11 @@ arv_gc_port_read (ArvGcPort *port, void *buffer, guint64 address, guint64 length
 			/* For schema < 1.1.0 and length == 4, register read must be used instead of memory read.
 			 * Only applies to GigE Vision devices. See Appendix 3 of Genicam 2.0 specification. */
 			if (ARV_IS_GV_DEVICE (device) && _use_legacy_endianess_mechanism (port, length)) {
-				guint32 value;
-
-				/* For schema < 1.1.0, all registers are big endian. */
-				value = *((guint32 *) buffer);
-				value = GUINT32_FROM_BE (value);
+				guint32 value = 0;
 
 				arv_device_read_register (device, address, &value, error);
+
+				/* For schema < 1.1.0, all registers are big endian. */
 				*((guint32 *) buffer) = GUINT32_TO_BE (value);
 			} else
 				arv_device_read_memory (device, address, length, buffer, error);
@@ -163,6 +162,8 @@ arv_gc_port_write (ArvGcPort *port, void *buffer, guint64 address, guint64 lengt
 
 	genicam = arv_gc_node_get_genicam (ARV_GC_NODE (port));
 
+	arv_gc_feature_node_increment_change_count (ARV_GC_FEATURE_NODE (port));
+
 	if (port->priv->chunk_id == NULL) {
 		device = arv_gc_get_device (genicam);
 
@@ -225,14 +226,14 @@ arv_gc_port_new (void)
 static void
 arv_gc_port_init (ArvGcPort *gc_port)
 {
-	gc_port->priv = G_TYPE_INSTANCE_GET_PRIVATE (gc_port, ARV_TYPE_GC_PORT, ArvGcPortPrivate);
+	gc_port->priv = arv_gc_port_get_instance_private (gc_port);
 	gc_port->priv->chunk_id = 0;
 }
 
 static void
 _finalize (GObject *object)
 {
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (arv_gc_port_parent_class)->finalize (object);
 }
 
 static void
@@ -241,20 +242,8 @@ arv_gc_port_class_init (ArvGcPortClass *this_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (this_class);
 	ArvDomNodeClass *dom_node_class = ARV_DOM_NODE_CLASS (this_class);
 
-#if !GLIB_CHECK_VERSION(2,38,0)
-	g_type_class_add_private (this_class, sizeof (ArvGcPortPrivate));
-#endif
-
-	parent_class = g_type_class_peek_parent (this_class);
-
 	object_class->finalize = _finalize;
 	dom_node_class->get_node_name = _get_node_name;
 	dom_node_class->post_new_child = _post_new_child;
 	dom_node_class->pre_remove_child = _pre_remove_child;
 }
-
-#if !GLIB_CHECK_VERSION(2,38,0)
-G_DEFINE_TYPE (ArvGcPort, arv_gc_port, ARV_TYPE_GC_FEATURE_NODE)
-#else
-G_DEFINE_TYPE_WITH_CODE (ArvGcPort, arv_gc_port, ARV_TYPE_GC_FEATURE_NODE, G_ADD_PRIVATE(ArvGcPort))
-#endif
