@@ -1,6 +1,6 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2012 Emmanuel Pacaud
+ * Copyright © 2009-2019 Emmanuel Pacaud
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Emmanuel Pacaud <emmanuel@gnome.org>
  */
@@ -29,6 +29,7 @@
 #include <arvgcregisternode.h>
 #include <arvgcregister.h>
 #include <arvgcinteger.h>
+#include <arvgcfeaturenodeprivate.h>
 #include <arvgcport.h>
 #include <arvgc.h>
 #include <arvmisc.h>
@@ -36,7 +37,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-static GObjectClass *parent_class = NULL;
+static void arv_gc_struct_entry_node_register_interface_init (ArvGcRegisterInterface *interface);
+static void arv_gc_struct_entry_node_integer_interface_init (ArvGcIntegerInterface *interface);
+
+G_DEFINE_TYPE_WITH_CODE (ArvGcStructEntryNode, arv_gc_struct_entry_node, ARV_TYPE_GC_FEATURE_NODE,
+			 G_IMPLEMENT_INTERFACE (ARV_TYPE_GC_REGISTER, arv_gc_struct_entry_node_register_interface_init)
+			 G_IMPLEMENT_INTERFACE (ARV_TYPE_GC_INTEGER, arv_gc_struct_entry_node_integer_interface_init))
 
 /* ArvDomNode implementation */
 
@@ -56,7 +62,6 @@ arv_gc_struct_entry_node_post_new_child (ArvDomNode *self, ArvDomNode *child)
 
 		switch (arv_gc_property_node_get_node_type (property_node)) {
 			case ARV_GC_PROPERTY_NODE_TYPE_SIGN:
-				/* TODO */
 				node->sign = property_node;
 				break;
 			case ARV_GC_PROPERTY_NODE_TYPE_LSB:
@@ -70,7 +75,7 @@ arv_gc_struct_entry_node_post_new_child (ArvDomNode *self, ArvDomNode *child)
 				node->lsb = property_node;
 				break;
 			default:
-				ARV_DOM_NODE_CLASS (parent_class)->post_new_child (self, child);
+				ARV_DOM_NODE_CLASS (arv_gc_struct_entry_node_parent_class)->post_new_child (self, child);
 				break;
 		}
 	}
@@ -138,6 +143,22 @@ _get_msb (ArvGcStructEntryNode *gc_struct_entry_node, GError **error)
 	return arv_gc_property_node_get_int64 (gc_struct_entry_node->msb, error);
 }
 
+static ArvGcSignedness
+_get_signedness (ArvGcStructEntryNode *gc_struct_entry_node, GError **error)
+{
+	const char *signedness;
+
+	if (gc_struct_entry_node->sign == NULL)
+		return ARV_GC_SIGNEDNESS_UNSIGNED;
+
+	signedness = arv_gc_property_node_get_string (gc_struct_entry_node->sign, error);
+
+	if (g_strcmp0 (signedness, "Unsigned") == 0)
+		return ARV_GC_SIGNEDNESS_UNSIGNED;
+
+	return ARV_GC_SIGNEDNESS_SIGNED;
+}
+
 /**
  * arv_gc_struct_entry_node_new:
  *
@@ -162,7 +183,7 @@ arv_gc_struct_entry_node_init (ArvGcStructEntryNode *gc_struct_entry_node)
 static void
 arv_gc_struct_entry_node_finalize (GObject *object)
 {
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (arv_gc_struct_entry_node_parent_class)->finalize (object);
 }
 
 static void
@@ -172,8 +193,6 @@ arv_gc_struct_entry_node_class_init (ArvGcStructEntryNodeClass *this_class)
 	ArvDomNodeClass *dom_node_class = ARV_DOM_NODE_CLASS (this_class);
 	ArvGcFeatureNodeClass *gc_feature_node_class = ARV_GC_FEATURE_NODE_CLASS (this_class);
 
-	parent_class = g_type_class_peek_parent (this_class);
-
 	object_class->finalize = arv_gc_struct_entry_node_finalize;
 	dom_node_class->get_node_name = arv_gc_struct_entry_node_get_node_name;
 	dom_node_class->post_new_child = arv_gc_struct_entry_node_post_new_child;
@@ -202,6 +221,7 @@ arv_gc_struct_entry_node_set (ArvGcRegister *gc_register, void *buffer, guint64
 
 	g_return_if_fail (ARV_IS_GC_REGISTER (struct_register));
 
+	arv_gc_feature_node_increment_change_count (ARV_GC_FEATURE_NODE (gc_register));
 	return arv_gc_register_set (ARV_GC_REGISTER (struct_register), buffer, length, error);
 }
 
@@ -259,6 +279,7 @@ arv_gc_struct_entry_node_get_integer_value (ArvGcInteger *gc_integer, GError **e
 {
 	ArvGcStructEntryNode *struct_entry = ARV_GC_STRUCT_ENTRY_NODE (gc_integer);
 	ArvDomNode *struct_register;
+	ArvGcSignedness signedness;
 	GError *local_error = NULL;
 	gint64 value;
 	guint lsb;
@@ -282,7 +303,15 @@ arv_gc_struct_entry_node_get_integer_value (ArvGcInteger *gc_integer, GError **e
 		return 0;
 	}
 
-	value = arv_gc_register_node_get_masked_integer_value (ARV_GC_REGISTER_NODE (struct_register), lsb, msb, &local_error);
+	signedness = _get_signedness (struct_entry, &local_error);
+
+	if (local_error != NULL) {
+		g_propagate_error (error, local_error);
+		return 0;
+	}
+
+	value = arv_gc_register_node_get_masked_integer_value (ARV_GC_REGISTER_NODE (struct_register),
+							       lsb, msb, signedness, &local_error);
 
 	if (local_error != NULL) {
 		g_propagate_error (error, local_error);
@@ -319,6 +348,7 @@ arv_gc_struct_entry_node_set_integer_value (ArvGcInteger *gc_integer, gint64 val
 		return;
 	}
 
+	arv_gc_feature_node_increment_change_count (ARV_GC_FEATURE_NODE (gc_integer));
 	arv_gc_register_node_set_masked_integer_value (ARV_GC_REGISTER_NODE (struct_register), lsb, msb, value, &local_error);
 
 	if (local_error != NULL) {
@@ -333,7 +363,3 @@ arv_gc_struct_entry_node_integer_interface_init (ArvGcIntegerInterface *interfac
 	interface->get_value = arv_gc_struct_entry_node_get_integer_value;
 	interface->set_value = arv_gc_struct_entry_node_set_integer_value;
 }
-
-G_DEFINE_TYPE_WITH_CODE (ArvGcStructEntryNode, arv_gc_struct_entry_node, ARV_TYPE_GC_FEATURE_NODE,
-			 G_IMPLEMENT_INTERFACE (ARV_TYPE_GC_REGISTER, arv_gc_struct_entry_node_register_interface_init)
-			 G_IMPLEMENT_INTERFACE (ARV_TYPE_GC_INTEGER, arv_gc_struct_entry_node_integer_interface_init))
